/* Se dispone de una lista simple cuyos nodos tienen la siguiente estructura:
 *	struct dato {
 *		char Alumno[20];
 *		char Clave[20];
 *	};
 *	struct nodo {
 *		struct dato d;
 *		struct nodo *sig;
 *	};
 *
 * 1) Realizar una función que elimine de la lista antes mencionada los nodos
 * que contengan el valor del campo "Clave" previamente seleccionado.
 *	Prototipo:
 *		void Filtrar(struct nodo **H, char *Filtro);
 * Ej: Si filtro apunta a "PEPE", se deberán eliminar todos los nodos en donde
 * "Clave" señale a strings de contenido "PEPE".
 *
 * 2) Guardar en un archivo llamado salida.txt la información de la lista resultante.
 *
 * 3) Realizar una función que libere los recursos utilizados por la lista.
 *	Prototipo:
 *		void Liberar(struct nodo **H);
 *
 * 4) Realizar un programa que asuma que la lista fue cargada por la función
 * void Cargar(struct nodo **); y utilice la función del punto 1 para realizar el
 * filtrado de los repetidos y la del punto 3 para liberar la lista. La clave debe
 * ser ingresada por línea de comandos.
 */

 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
 
 struct dato {
	char Alumno[20];
	char Clave[20];
};

struct nodo {
	struct dato d;
	struct nodo *sig;
};

void Cargar(struct nodo **);
void Filtrar(struct nodo **, char *);
int guardarlista(struct nodo **);
void Liberar(struct nodo **);

int main(int argc, char *argv[])
{
	struct nodo *H=NULL;
	
	if(argc < 2) {
		fprintf(stderr, "No especificó filtro, saliendo.\n");
		return -1;
	}
	
	Cargar(&H);
	Filtrar(&H);
	if(guardarlista(&H)==-2) {
		perror("Error abriendo archivo de salida");
		return -1;
	}
	Liberar(&H);
	
	return 0;
}

void Filtrar(struct nodo **H, char *filtro)
{
	struct nodo *q = *H, *p = NULL;
	
	if(!(*H)) return; //Lista vacía.
	
	while(q) { //Mientras que q no sea NULL...
		if(!strcmp(q->d.Clave, filtro)) {
			if(!p) { //p es NULL, por lo tanto es el primer nodo.
				(*H) = q->sig; //Corro la cabecera de la lista al siguiente nodo
				free(q); //y simplemente lo libero.
				q = *H; //Cargo el siguiente.
			}
			else { //Estoy en el medio de la lista.
				p->sig = q->sig; //p es el anterior nodo que evalué, lo encadeno al 
				free(q); //siguiente nodo, y lo libero.
			}
		}
		else { //Las claves no coinciden.
			p = q; //Cargo en p el anterior nodo
			q = q->sig; //y avanzo.
		}
	}
	
	return;
}

int guardarlista(struct nodo **H)
{
	FILE *file;
	struct nodo *entrada;
	
	if(!(*H)) return -1; //lista vacía
	file = fopen("salida.txt", "w");
	if(!file) return -2; //error abriendo archivo
	
	entrada = *H;
	while(entrada) {
		fprintf(file, "Alumno: %s, Clave: %s\n", entrada->d.Alumno, entrada->d.Clave);
		entrada = entrada->sig;
	}
	
	fclose(file);
	return 0;
}

void Liberar(struct nodo **H)
{
	struct nodo *q = *H, *p = NULL;
	
	if(!(*H)) return; //lista vacía
	
	while(q) {
		p = q->sig; //Cargo en p el siguiente nodo
		free(q); //Libero q
		q = p; //Cargo en q el valor de p
	}
	
	*H = NULL;
	return;
}
